// main.cpp : Defines the entry point for the console application.
//

#include <fstream>
#include <Cassert>
#include <iostream>

#include "Aluno.h"

#define TAMANHO_BUFFER		99999

struct Slot
{
	enum Estado {NOVO = 0, OCUPADO, VAZIO};

	int estado;
	Aluno aluno;

	Slot () : estado (NOVO) {}
};

struct Bucket
{
	int controle;
	Slot slots[NUM_SLOTS];
};

void extrairAluno( const string &_description, Aluno & aluno );
bool imprimir (const string & _alunosFileName, const string & _printFileName);

using namespace std;

int main(int argc, char* argv[])
{
    int i;
	Aluno aluno;
	Bucket bucket;
	fstream fstex, fsbin;
    
	fstex.open ( "../EDIIA32.txt", ios::in );

	if ( fstex.fail( ) )
		return 1;

	fsbin.open ( "../AcessoDireto.dsk", ios::out | ios::binary );

	if ( fsbin.fail( ) )
		return false;

	do
	{
		char buffer[ TAMANHO_BUFFER ];
		
//		L a prxima linha do arquivo.
		fstex.getline( buffer, TAMANHO_BUFFER );

//		Verifica se  uma linha vlida.
		if( string( buffer ).empty( ) )
			break;

//		Preenche os dados do aluno a partir da linha lida do arquivo.
		extrairAluno( buffer, aluno );

//		Imprime na tela os dados do aluno.
		printf ( "%s\n", aluno.toString( ).c_str ( ) );

//------ Grava o registro do aluno no arquivo binrio.
		for ( i = 0; i < NUM_SLOTS; i++ )
		{
//			Verifica se este slot no est ocupado.
			if ( bucket.slots[i].estado != Slot::OCUPADO ) 
			{
//				Caso verdadeiro, armazena o aluno neste slot.
				bucket.slots[i].aluno = aluno;
				bucket.slots[i].estado = Slot::OCUPADO;
				break;
			}
		}

//		Verifica se todos os slots foram preenchidos.
		if ( i == NUM_SLOTS-1 ) 
		{
//			Caso verdadeiro, escreve este bucket no arquivo.
			fsbin.write( (char *) &bucket, sizeof (Bucket) );

//			e limpa o contedo do bucket.
			for ( i = 0; i < NUM_SLOTS; i++ )
				bucket.slots[i].estado = Slot::VAZIO;
		}
//------ 

	} while( !fstex.fail( ) && !fsbin.fail( ) );

	fstex.close ( );
	fsbin.close ( );

	if ( !imprimir ( "../AcessoDireto.dsk", "../EDIIA33.TXT" ) )
		return 1;

	getchar ( );

	return 0;
}

// ____________________________________________________________________________
//
// Preenche os dados de um aluno a partir de um CString.
//

void extrairAluno( const string &_description, Aluno & aluno )
{
	int index = 0;

	const string strMatricula = _description.substr( index, COMPRIMENTO_MAT );

	index += COMPRIMENTO_MAT;

	const string strNome = _description.substr( index, COMPRIMENTO_NOME );

	index += COMPRIMENTO_NOME;

	const string strEndereco = _description.substr( index, COMPRIMENTO_END );

	index += COMPRIMENTO_END;

	const string strPeriodo = _description.substr( index, 1 );

	aluno.setMatricula	( strMatricula.c_str ( )		);
	aluno.setNome		( strNome.c_str ( )				);
	aluno.setEndereco	( strEndereco.c_str ( )			);
	aluno.setPeriodo	( atoi( strPeriodo.c_str ( ) )	);
}

// ____________________________________________________________________________
//
// A partir de um arquivo binrio, imprime na tela os alunos identificados 
// pelos pares (slot bucket), lidos a partir de um arquivo texto.
//
//
bool imprimir (const string & _filenamebin, const string & _filenametex)
{
	int nBucket, nSlot;
	Bucket bucket;
	fstream fsbin, fstex;
	
	fsbin.open ( _filenamebin.c_str ( ), ios::in | ios::binary );
	fstex.open ( _filenametex.c_str ( ), ios::in );

	if ( fsbin.fail( ) || fstex.fail( ) )
		return false;

	printf ( "\n" );
	
	do
	{
		// 
		fstex >> nSlot >> nBucket;
		
		if( !fstex )
			break;
 
 //   calcula a posicao de nBucket, em bytes, em relacao ao inicio do arquivo 
		ios::pos_type pos = (nBucket-1) * sizeof (Bucket);
 
 //   ajusta o ponteiro para o endereco desejado		
		fsbin.seekg ( pos, ios::beg );

 //   le do arquivo sizeof (Bucket) bytes para a referencia &bucket
		fsbin.read ( (char *) &bucket, sizeof (Bucket) );

        printf ("Bucket %d \t Slot %d \n",  nBucket, nSlot);
        printf ("%s \n", bucket.slots[nSlot-1].aluno.toString( ).c_str ( ) );
	} while( !fstex.fail( ) );

	fstex.close ( );
	fsbin.close ( );

    getchar ( );
    
	return true;
}
